unit UGoal;

// A "TGoal" is a mechanism to compare 2 states and get a "Score" to be
// added to the "owner", a TPlayer or a TTeam.
//
// When used on a TPlayer, the "Goal" will score each step of the future
// so the future's score will be a sum. of easch step's goal's score.
//
// When used on a TTeam it will add TeamScore to all player's futures by comparing
// the initial state of the player with the end state of the future.

interface

uses UState, ExtCtrls;


type TGoal = class(TObject)
  private
    // How strong the goal has to be?
    fCoef: double;
    // How much score did I add to the player last time?
    fScore: double;
  protected
    // Can this goal be drawed on screen?
    fDrawable: boolean;
    // Need to be redraw for each player connected or one draw is ok?
    // Example: A rotational goal draw a line from the center to the player,
    // but a "pick up sweets" goal can be draw one time for all players.
    fPersonalizedDrawing: boolean;
  public
    constructor create(c: double); virtual;
    procedure recreate; virtual;
    // THIS is the procedure you need to redefine on your instance!
    procedure ScoreGoal(player: TObject); virtual;
    procedure AddScore(player: TObject);
    procedure Draw(player: TObject); virtual; abstract;
    property  Coef: double read fCoef write fCoef;
    property  Score: double read fScore write fScore;
    property  Drawable: boolean read fDrawable;
    property  PersonalizedDrawing: boolean read fPersonalizedDrawing;
  end;


implementation

uses UPlayer;


constructor TGoal.create(c: double);
begin
  inherited create;
  fDrawable:= false;
  fPersonalizedDrawing:= true;
  fScore:= 0;
  fCoef:=  c;
end;

// If you change sometinhg (the track) may be you need to re-create the goal
procedure TGoal.recreate;
begin
  // Define it on you TGoal descendant if needed.
  // ...
end;

procedure TGoal.ScoreGoal(player: TObject);
begin
  Score:= 0;
  // Scoring will go here on descendant classes.
  // ...
end;

procedure TGoal.AddScore(player: TObject);
begin
  // Add it to the player's MyScore:
  TPlayer(player).State.MyScore:= TPlayer(player).State.MyScore + (Coef * Score);
  // Reset it (for cumulative goals):
  Score:= 0;
end;

end.
